import re
from setuptools import setup

try:
  from pypandoc import convert
  read_md = lambda f: convert(f, 'rst')
except ImportError:
  print('Warning: pypandoc module not found, could not convert Markdown to RST')
  read_md = lambda f: open(f, 'r').read()

with open('./merge_ics/__init__.py') as f:
  version = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", f.read(), re.M).group(1)

setup(
  name = 'merge_ics',
  version = version,
  author = 'Alberto Garcia-Raboso',
  author_email = 'agraboso@gmail.com',
  description = 'Merge RFC5545 calendar files',
  keywords = 'RFC5545 ics calendar',
  license = 'MIT',
  url = 'https://github.com/agraboso/merge_ics',
  platforms = ['any'],
  install_requires = [
    'PyYAML >3',
    'requests >=2.7.0',
    'icalendar >=3.9.0, <=4'
    ],
  packages = ['merge_ics'],
  entry_points = {
    'console_scripts': ['merge_ics = merge_ics.merge_ics:main']
    },
  include_package_data = True,
  long_description = read_md('README.md'),
  classifiers = [
    'Programming Language :: Python',
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Operating System :: OS Independent',
    'Natural Language :: English',
    'Topic :: Utilities',
    'License :: OSI Approved :: MIT License'
    ]
  )



import sys
import yaml
import requests
from icalendar import Calendar, Event

in_cals = {}

def read_config(filename):
  """Read YAML config file"""
  with open(filename, 'r') as f:
    return yaml.load(f)

def download_calendar(url):
  """Download and parse ics file"""
  req = requests.get(url)
  cal = Calendar.from_ical(req.text)
  return cal

def read_calendar(filename):
  """Parse local ics file"""
  with open(filename, 'r') as f:
    cal = Calendar.from_ical(f)
  return cal

def write_calendar(options, sources, filename):
  """Create and write ics file"""
  cal = Calendar()
  timezones_cache = []
  for key, value in options.items():
    cal.add(key, value)
  for source_id, category in sources.items():
    for timezone in in_cals[source_id].walk('VTIMEZONE'):
      if timezone['tzid'] not in timezones_cache:
        timezones_cache.append(comp['tzid'])
        cal.add_component(timezone)
    for event in in_cals[source_id].walk('VEVENT'):
      event_copy = Event(event)
      event_copy.add('categories', category)
      cal.add_component(event_copy)
  with open(filename, 'w') as f:
    f.write(cal.to_ical())

def main():
  # Get the name of the config file
  if len(sys.argv) != 2:
    print 'Usage:'
    print '  python merge-ics.py <config_file>'
    return 1
  config_file = sys.argv[1]

  # Read config
  try:
    config = read_config(config_file)
  except IOError:
    print 'Unable to open ' + config_file
    return 1
  except yaml.YAMLError:
    print 'Unable to parse ' + config_file
    return 1

  # Read/download and parse input calendars
  for source_id, source in config['sources'].items():
    if 'filename' in source:
      try:
        in_cals[source_id] = read_calendar(source['filename'])
      except IOError:
        print 'Unable to open ' + source['filename']
      except ValueError:
        print 'Unable to parse ' + source['filename']
    if (source_id not in in_cals) and ('url' in source):
      try:
        in_cals[source_id] = download_calendar(source['url'])
      except requests.exceptions.RequestException:
        print 'Unable to download ' + source['url']
      except ValueError:
        print 'Unable to parse ' + source['url']

  # Create and write output calendars
  for sink in config['sinks']:
    try:
      write_calendar(sink['options'], sink['sources'], sink['filename'])
    except IOError:
      print 'Unable to write ' + sink['filename']
    except ValueError:
      print 'Unable to create calendar ' + sink['filename']
